/**
 * Copyright (C) 2011 Angelo Zerr <angelo.zerr@gmail.com> and Pascal Leclercq <pascal.leclercq@gmail.com>
 *
 * All rights reserved.
 *
 * Permission is hereby granted, free  of charge, to any person obtaining
 * a  copy  of this  software  and  associated  documentation files  (the
 * "Software"), to  deal in  the Software without  restriction, including
 * without limitation  the rights to  use, copy, modify,  merge, publish,
 * distribute,  sublicense, and/or sell  copies of  the Software,  and to
 * permit persons to whom the Software  is furnished to do so, subject to
 * the following conditions:
 *
 * The  above  copyright  notice  and  this permission  notice  shall  be
 * included in all copies or substantial portions of the Software.
 *
 * THE  SOFTWARE IS  PROVIDED  "AS  IS", WITHOUT  WARRANTY  OF ANY  KIND,
 * EXPRESS OR  IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE  WARRANTIES OF
 * MERCHANTABILITY,    FITNESS    FOR    A   PARTICULAR    PURPOSE    AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE,  ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package fr.opensagres.xdocreport.document.docx;

import fr.opensagres.xdocreport.converter.MimeMapping;
import fr.opensagres.xdocreport.core.document.DocumentKind;
import fr.opensagres.xdocreport.core.io.XDocArchive;
import fr.opensagres.xdocreport.document.AbstractXDocReport;
import fr.opensagres.xdocreport.document.docx.preprocessor.DocXPreprocessor;

/**
 * MS Word DOCX report.
 * 
 */
public class DocXReport extends AbstractXDocReport implements DocXConstants {

	private static final long serialVersionUID = -2323716817951928168L;

	private static final String[] DEFAULT_XML_ENTRIES = {
			WORD_DOCUMENT_XML_ENTRY, WORD_STYLES_XML_ENTRY,
			WORD_HEADER_XML_ENTRY, WORD_FOOTER_XML_ENTRY };

	private static final String WORD_REGEXP = "word*";

	public String getKind() {
		return DocumentKind.DOCX.name();
	}

	@Override
	protected void registerPreprocessors() {
		super.addPreprocessor(WORD_DOCUMENT_XML_ENTRY,
				DocXPreprocessor.INSTANCE);
		super.addPreprocessor(WORD_HEADER_XML_ENTRY,
				DocXPreprocessor.INSTANCE);		
		super.addPreprocessor(WORD_FOOTER_XML_ENTRY,
				DocXPreprocessor.INSTANCE);		
	}

	@Override
	protected String[] getDefaultXMLEntries() {
		return DEFAULT_XML_ENTRIES;
	}

	// @Override
	// protected String[] getDefaultXMLEntriesForConversion() {
	// return DEFAULT_XML_ENTRIES_FOR_CONVERSION;
	// }

	public MimeMapping getMimeMapping() {
		return MIME_MAPPING;
	}

	public static boolean isDocx(XDocArchive documentArchive) {
		if (!documentArchive.hasEntry(CONTENT_TYPES_XML)) {
			return false;
		}
		// TODO check from [Content_Types].xml
		// <Override PartName="/word/document.xml"
		// ContentType="application/vnd.openxmlformats-officedocument.wordprocessingml.document.main+xml"
		// />
		return documentArchive.getEntryNames(WORD_REGEXP).size() > 0;
	}

}
